@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id: ephy-history.c 6952 2007-03-11 19:42:02Z chpe $
+ *  $Id: ephy-history.c 7858 2008-01-13 20:42:01Z cosimoc $
  */
 
 #include "config.h"
@@ -30,11 +30,11 @@
 #include "eel-gconf-extensions.h"
 #include "ephy-prefs.h"
 #include "ephy-glib-compat.h"
+#include "ephy-string.h"
 
 #include <time.h>
 #include <string.h>
 #include <glib/gi18n.h>
-#include <libgnomevfs/gnome-vfs-uri.h>
 
 #define EPHY_HISTORY_XML_ROOT	 (const xmlChar *)"ephy_history"
 #define EPHY_HISTORY_XML_VERSION (const xmlChar *)"1.0"
@@ -84,7 +84,8 @@ enum
 	VISITED,
 	CLEARED,
 	REDIRECT,
-        LAST_SIGNAL
+	ICON_UPDATED,
+	LAST_SIGNAL
 };
 
 static guint signals[LAST_SIGNAL] = { 0 };
@@ -124,22 +125,6 @@ ephy_history_get_type (void)
         return type;
 }
 
-static void
-ephy_history_set_enabled (EphyHistory *history,
-			  gboolean enabled)
-{
-	LOG ("ephy_history_set_enabled %d", enabled);
-
-	history->priv->enabled = enabled;
-
-	ephy_node_db_set_immutable (history->priv->db, !enabled);
-
-	if (enabled == FALSE)
-	{
-		ephy_history_clear (history);
-	}
-}
-
 static void
 ephy_history_set_property (GObject *object,
 			   guint prop_id,
@@ -169,7 +154,7 @@ ephy_history_get_property (GObject *object,
 		case PROP_ENABLED:
 			g_value_set_boolean (value, history->priv->enabled);
 			break;
-		}
+	}
 }
 
 static void
@@ -239,6 +224,17 @@ ephy_history_class_init (EphyHistoryClass *klass)
 			      G_TYPE_STRING | G_SIGNAL_TYPE_STATIC_SCOPE,
 			      G_TYPE_STRING | G_SIGNAL_TYPE_STATIC_SCOPE);
 
+	signals[ICON_UPDATED] =
+		g_signal_new ("icon_updated",
+			      G_OBJECT_CLASS_TYPE (object_class),
+			      G_SIGNAL_RUN_FIRST,
+			      G_STRUCT_OFFSET (EphyHistoryClass, icon_updated),
+			      NULL, NULL,
+			      ephy_marshal_VOID__STRING_STRING,
+			      G_TYPE_NONE,
+			      2,
+			      G_TYPE_STRING, G_TYPE_STRING);
+
 	g_type_class_add_private (object_class, sizeof (EphyHistoryPrivate));
 }
 
@@ -288,7 +284,7 @@ ephy_history_save (EphyHistory *eb)
 	int ret;
 
 	/* only save if there are changes */
-	if (eb->priv->dirty == FALSE)
+	if (eb->priv->dirty == FALSE && eb->priv->enabled)
 	{
 		return;
 	}
@@ -662,11 +658,10 @@ ephy_history_host_visited (EphyHistory *eh,
 static EphyNode *
 internal_get_host (EphyHistory *eh, const char *url, gboolean create)
 {
-	GnomeVFSURI *vfs_uri = NULL;
 	EphyNode *host = NULL;
-	const char *host_name = NULL;
+	char *host_name = NULL;
 	GList *host_locations = NULL, *l;
-	const char *scheme = NULL;
+	char *scheme = NULL;
 	GTime now;
 
 	g_return_val_if_fail (url != NULL, NULL);
@@ -678,24 +673,22 @@ internal_get_host (EphyHistory *eh, const char *url, gboolean create)
 
 	now = time (NULL);
 
-	vfs_uri = gnome_vfs_uri_new (url);
-
-	if (vfs_uri)
+	if (url)
 	{
-		scheme = gnome_vfs_uri_get_scheme (vfs_uri);
-		host_name = gnome_vfs_uri_get_host_name (vfs_uri);
+		scheme = g_uri_get_scheme (url);
+		host_name = ephy_string_get_host_name (url);
 	}
 
 	/* Build an host name */
 	if (scheme == NULL || host_name == NULL)
 	{
-		host_name = _("Others");
+		host_name = g_strdup (_("Others"));
 		host_locations = g_list_append (host_locations,
 						g_strdup ("about:blank"));
 	}
 	else if (strcmp (scheme, "file") == 0)
 	{
-		host_name = _("Local files");
+		host_name = g_strdup (_("Local files"));
 		host_locations = g_list_append (host_locations,
 						g_strdup ("file:///"));
 	}
@@ -703,23 +696,34 @@ internal_get_host (EphyHistory *eh, const char *url, gboolean create)
 	{
 		char *location;
 		char *tmp;
+		
+		if (g_str_equal (scheme, "https"))
+		{
+			/* If scheme is https, we still fake http. */
+			location = g_strconcat ("http://", host_name, "/", NULL);
+			host_locations = g_list_append (host_locations, location);
+		}
 
+		/* We append the real address */
 		location = g_strconcat (scheme,
 					"://", host_name, "/", NULL);
 		host_locations = g_list_append (host_locations, location);
 
-		if (g_str_has_prefix (host_name, "www."))
+		/* and also a fake www-modified address if it's http or https. */
+		if (g_str_has_prefix (scheme, "http"))
 		{
-			tmp = g_strdup (g_utf8_offset_to_pointer (host_name, 4));
+			if (g_str_has_prefix (host_name, "www."))
+			{
+				tmp = g_strdup (host_name + 4);
+			}
+			else
+			{
+				tmp = g_strconcat ("www.", host_name, NULL);
+			}
+			location = g_strconcat ("http://", tmp, "/", NULL);
+			g_free (tmp);
+			host_locations = g_list_append (host_locations, location);
 		}
-		else
-		{
-			tmp = g_strconcat ("www.", host_name, NULL);
-		}
-		location = g_strconcat (gnome_vfs_uri_get_scheme (vfs_uri),
-					"://", tmp, "/", NULL);
-		g_free (tmp);
-		host_locations = g_list_append (host_locations, location);
 	}
 
 	g_return_val_if_fail (host_locations != NULL, NULL);
@@ -759,10 +763,8 @@ internal_get_host (EphyHistory *eh, const char *url, gboolean create)
 		ephy_history_host_visited (eh, host, now);
 	}
 
-	if (vfs_uri)
-	{
-		gnome_vfs_uri_unref (vfs_uri);
-	}
+	g_free (scheme);
+	g_free (host_name);
 
 	g_list_foreach (host_locations, (GFunc)g_free, NULL);
 	g_list_free (host_locations);
@@ -966,16 +968,23 @@ ephy_history_set_icon (EphyHistory *gh,
 		       const char *url,
 		       const char *icon)
 {
-	EphyNode *host;
+	EphyNode *node, *host;
+	int host_id;
 
-	LOG ("Set host icon");
+	node = ephy_history_get_page (gh, url);
+	if (node == NULL) return;
+	
+	host_id = ephy_node_get_property_int (node, EPHY_NODE_PAGE_PROP_HOST_ID);
+	g_return_if_fail (host_id >= 0);
 
-	host = g_hash_table_lookup (gh->priv->hosts_hash, url);
+	host = ephy_node_db_get_node_from_id (gh->priv->db, host_id);
 	if (host)
 	{
 		ephy_node_set_property_string (host, EPHY_NODE_PAGE_PROP_ICON,
 					       icon);
 	}
+
+	g_signal_emit (gh, signals[ICON_UPDATED], 0, url, icon);
 }
 
 void
@@ -1026,3 +1035,39 @@ ephy_history_is_enabled (EphyHistory *history)
 
 	return history->priv->enabled;
 }
+
+void
+ephy_history_set_enabled (EphyHistory *history,
+			  gboolean enabled)
+{
+	int ret;
+
+	ret = 1;
+
+	LOG ("ephy_history_set_enabled %d", enabled);
+
+	/* Write history only when disabling it, not when reenabling it */
+	if (!enabled && history->priv->dirty)
+	{
+		ret = ephy_node_db_write_to_xml_safe
+			(history->priv->db, (const xmlChar *)history->priv->xml_file,
+			 EPHY_HISTORY_XML_ROOT,
+			 EPHY_HISTORY_XML_VERSION,
+			 NULL, /* comment */
+			 history->priv->hosts,
+			 (EphyNodeFilterFunc) save_filter, history->priv->pages,
+			 history->priv->pages, NULL, NULL,
+			 NULL);
+	}
+
+	if (ret >=0)
+	{
+		/* save was successful */
+		history->priv->dirty = FALSE;
+	}
+
+	history->priv->enabled = enabled;
+
+	ephy_node_db_set_immutable (history->priv->db, !enabled);
+}
+
