@@ -36,14 +36,13 @@
  *
  * ***** END LICENSE BLOCK *****
  *
- * $Id: EphyHeaderSniffer.cpp 7426 2007-09-13 13:18:57Z chpe $
+ * $Id: EphyHeaderSniffer.cpp 7858 2008-01-13 20:42:01Z cosimoc $
  */
 
 #include "mozilla-config.h"
 #include "config.h"
 
 #include <glib/gi18n.h>
-#include <libgnomevfs/gnome-vfs-utils.h>
 
 #include <nsStringAPI.h>
 
@@ -73,7 +72,9 @@
 #include "ephy-gui.h"
 #include "ephy-prefs.h"
 
+#ifndef HAVE_GECKO_1_9
 #include "EphyBadCertRejector.h"
+#endif
 #include "MozDownload.h"
 
 #include "EphyHeaderSniffer.h"
@@ -223,6 +224,7 @@ EphyHeaderSniffer::OnSecurityChange (nsIWebProgress *aWebProgress, nsIRequest *a
 NS_IMETHODIMP
 EphyHeaderSniffer::GetInterface(const nsIID & uuid, void * *result)
 {
+#ifndef HAVE_GECKO_1_9
 	if (uuid.Equals (NS_GET_IID (nsIBadCertListener)) &&
 	    mEmbedPersist)
 	{
@@ -241,6 +243,7 @@ EphyHeaderSniffer::GetInterface(const nsIID & uuid, void * *result)
 			return NS_OK;
 		}
 	}
+#endif
 
 	return NS_ERROR_NO_INTERFACE;
 }
@@ -378,8 +381,7 @@ nsresult EphyHeaderSniffer::PerformSave (nsIURI* inOriginalURI)
 	/* FIXME: do better here by using nsITextToSubURI service, like in
 	 * http://lxr.mozilla.org/seamonkey/source/xpfe/communicator/resources/content/contentAreaUtils.js#763
 	 */
-        char *filename;
-        filename = gnome_vfs_unescape_string (default_name, NULL);
+        char *filename = g_uri_unescape_string (default_name, NULL);
 
         if (!g_utf8_validate (filename, -1, NULL))
         {
