@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2004 Jean-François rameau
+ *  Copyright © 2004 Jean-François rameau
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -13,9 +13,9 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id: ephy-dbus.h,v 1.3 2006/01/27 21:32:27 chpe Exp $
+ *  $Id: ephy-dbus.h 6952 2007-03-11 19:42:02Z chpe $
  */
 
 #ifndef EPHY_DBUS_H
@@ -30,11 +30,6 @@
 
 G_BEGIN_DECLS
 
-/* Epiphany's DBUS ids */
-#define DBUS_EPHY_SERVICE		"org.gnome.Epiphany"
-#define DBUS_EPHY_PATH			"/org/gnome/Epiphany"
-#define DBUS_EPHY_INTERFACE		"org.gnome.Epiphany"
-
 #define EPHY_TYPE_DBUS		(ephy_dbus_get_type ())
 #define EPHY_DBUS(o)		(G_TYPE_CHECK_INSTANCE_CAST ((o), EPHY_TYPE_DBUS, EphyDbus))
 #define EPHY_DBUS_CLASS(k)	(G_TYPE_CHECK_CLASS_CAST((k), EPHY_TYPE_DBUS, EphyDbusClass))
@@ -85,7 +80,8 @@ DBusGProxy	*ephy_dbus_get_proxy	(EphyDbus *dbus,
 					 EphyDbusBus kind);
 
 /* private */
-gboolean       _ephy_dbus_startup	(GError **error);
+gboolean       _ephy_dbus_startup	(gboolean claim_name,
+					 GError **error);
 
 void	       _ephy_dbus_release	(void);
 
