@@ -14,9 +14,9 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id: ephy-main.c 6940 2007-03-05 19:17:20Z chpe $
+ *  $Id: ephy-main.c 7858 2008-01-13 20:42:01Z cosimoc $
  */
 
 #include "config.h"
@@ -47,8 +47,6 @@
 #include <libgnome/gnome-program.h>
 #include <libgnomeui/gnome-ui-init.h>
 
-#include <libgnomevfs/gnome-vfs-init.h>
-#include <libgnomevfs/gnome-vfs-utils.h>
 #include <libgnomeui/gnome-app-helper.h>
 
 #include <errno.h>
@@ -203,11 +201,24 @@ handle_email (GtkAboutDialog *about,
 	      const char *link,
 	      gpointer data)
 {
-	char *address;
+	char *command, *handler;
+	GAppInfo *appinfo;
 
-	address = g_strdup_printf ("mailto:%s", link);
-	gnome_vfs_url_show (address);
-	g_free (address);
+	if (eel_gconf_get_boolean ("/desktop/gnome/url-handlers/mailto/enabled") == FALSE)
+	{
+		return;
+	}
+	/* FIXME: better use g_app_info_get_default_for_uri_scheme () when it is
+	 * implemented.
+	 */
+	handler = eel_gconf_get_string ("/desktop/gnome/url-handlers/mailto/command");
+	command = g_strconcat (handler, "mailto:", link, NULL);
+	appinfo = g_app_info_create_from_commandline (command, NULL, 0, NULL);
+	ephy_file_launch_application (appinfo, NULL,
+				      gtk_get_current_event_time (),
+				      GTK_WIDGET (about));
+	g_free (handler);
+	g_free (command);
 }
 
 static void
@@ -369,7 +380,7 @@ queue_commands (guint32 user_time)
 		}
 		if (open_in_new_tab)
 		{
-			g_string_append (options, "new-tab,");
+			g_string_append (options, "new-tab,external,");
 		}
 
 		ephy_session_queue_command (session,
@@ -504,6 +515,9 @@ main (int argc,
 					   N_("GNOME Web Browser"),
 					   N_("GNOME Web Browser options"),
 					   NULL, NULL);
+
+	g_option_group_set_translation_domain (option_group, GETTEXT_PACKAGE);
+
 	g_option_group_add_entries (option_group, option_entries);
 
 	g_option_context_set_main_group (option_context, option_group);
@@ -582,12 +596,15 @@ main (int argc,
 			{
 				g_free (arguments[i]);
 
-				/* If it's a file, use gnome_vfs_make_uri_from_shell_arg,
+				/* If it's a file, use g_file_new_for_commandline_arg,
 				 * so we get the right escaping.
 				 */
 				if (path != NULL)
 				{
-					arguments[i] = gnome_vfs_make_uri_from_shell_arg (uri);
+					GFile *file;
+					file = g_file_new_for_commandline_arg (uri);
+					arguments[i] = g_file_get_uri (file);
+					g_object_unref (file);
 					g_free (uri);
 				}
 				else
@@ -672,7 +689,7 @@ main (int argc,
 
 	if (!ephy_file_helpers_init (profile_directory,
 				     private_instance,
-				     keep_temp_directory,
+				     keep_temp_directory || profile_directory,
 				     &error))
 	{
 		_ephy_dbus_release ();
@@ -724,7 +741,6 @@ main (int argc,
 	gnome_accelerators_sync ();
 	ephy_state_save ();
 	ephy_file_helpers_shutdown ();
-	gnome_vfs_shutdown ();
 	xmlCleanupParser ();
 
 	_ephy_dbus_release ();
