@@ -1,3 +1,4 @@
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
  *  Copyright © 2002 Jorn Baayen
  *  Copyright © 2003, 2004 Marco Pesenti Gritti
@@ -17,15 +18,15 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id: ephy-session.c 7554 2007-10-23 23:33:37Z cosimoc $
+ *  $Id: ephy-session.c 7858 2008-01-13 20:42:01Z cosimoc $
  */
 
 #include "config.h"
 
 #include "ephy-session.h"
 
+#include "ephy-embed-container.h"
 #include "ephy-window.h"
-#include "ephy-tab.h"
 #include "ephy-shell.h"
 #include "ephy-history-window.h"
 #include "ephy-bookmarks-editor.h"
@@ -39,6 +40,7 @@
 #include "ephy-notebook.h"
 
 #include <glib/gi18n.h>
+#include <gio/gio.h>
 #include <gtk/gtkmain.h>
 #include <gtk/gtkimage.h>
 #include <gtk/gtklabel.h>
@@ -51,9 +53,6 @@
 
 #include <libgnomeui/gnome-client.h>
 
-#include <libgnomevfs/gnome-vfs-ops.h>
-#include <libgnomevfs/gnome-vfs-utils.h>
-
 #include <libxml/tree.h>
 #include <libxml/xmlwriter.h>
 
@@ -390,10 +389,11 @@ die_cb (GnomeClient* client,
 
 /* Helper functions */
 
-static char *
-get_session_filename (const char *filename)
+static GFile *
+get_session_file (const char *filename)
 {
-	char *save_to;
+	GFile *file;
+	char *path;
 
 	if (filename == NULL)
 	{
@@ -402,58 +402,62 @@ get_session_filename (const char *filename)
 
 	if (strcmp (filename, SESSION_CRASHED) == 0)
 	{
-		save_to = g_build_filename (ephy_dot_dir (),
-					    "session_crashed.xml",
-					    NULL);
+		path = g_build_filename (ephy_dot_dir (),
+					 "session_crashed.xml",
+					 NULL);
 	}
 	else
 	{
-		save_to = g_strdup (filename);
+		path = g_strdup (filename);
 	}
 
-	return save_to;
+	file = g_file_new_for_path (path);
+	g_free (path);
+
+	return file;
 }
 
 static void
 session_delete (EphySession *session,
 		const char *filename)
 {
-	char *save_to;
+	GFile *file;
 
-	save_to = get_session_filename (filename);
+	file = get_session_file (filename);
 
-	gnome_vfs_unlink (save_to);
-
-	g_free (save_to);
+	g_file_delete (file, NULL, NULL);
 }
 
 static void
-net_stop_cb (EphyEmbed *embed,
-	     EphySession *session)
+load_status_notify_cb (EphyEmbed *embed,
+		       GParamSpec *pspec,
+		       EphySession *session)
 {
-	ephy_session_save (session, SESSION_CRASHED);
+	if (ephy_embed_get_load_status (embed) == FALSE)
+		ephy_session_save (session, SESSION_CRASHED);
 }
 
 static void
 notebook_page_added_cb (GtkWidget *notebook,
-			EphyTab *tab,
+			EphyEmbed *embed,
 			guint position,
 			EphySession *session)
 {
-	g_signal_connect (ephy_tab_get_embed (tab), "net-stop",
-			  G_CALLBACK (net_stop_cb), session);
+	g_signal_connect (embed, "notify::load-status",
+			  G_CALLBACK (load_status_notify_cb), session);
 }
 
 static void
 notebook_page_removed_cb (GtkWidget *notebook,
-			  EphyTab *tab,
+			  EphyEmbed *embed,
 			  guint position,
 			  EphySession *session)
 {
 	ephy_session_save (session, SESSION_CRASHED);
 
 	g_signal_handlers_disconnect_by_func
-		(ephy_tab_get_embed (tab), G_CALLBACK (net_stop_cb), session);
+		(embed, G_CALLBACK (load_status_notify_cb),
+		 session);
 }
 
 static void
@@ -527,7 +531,7 @@ resume_dialog_response_cb (GtkWidget *dialog,
 	if (response == GTK_RESPONSE_ACCEPT)
 	{
 		ephy_session_queue_command (session,
-				  	    EPHY_SESSION_CMD_LOAD_SESSION,
+					    EPHY_SESSION_CMD_LOAD_SESSION,
 					    SESSION_CRASHED, NULL,
 					    user_time, TRUE);
 	}
@@ -562,14 +566,17 @@ session_command_autoresume (EphySession *session,
 {
 	EphySessionPrivate *priv = session->priv;
 	GtkWidget *dialog;
-	char *saved_session;
+	GFile *saved_session_file;
+	char *saved_session_file_path;
 	gboolean crashed_session;
 
 	LOG ("ephy_session_autoresume");
 
-	saved_session = get_session_filename (SESSION_CRASHED);
-	crashed_session = g_file_test (saved_session, G_FILE_TEST_EXISTS);
-	g_free (saved_session);
+	saved_session_file = get_session_file (SESSION_CRASHED);
+	saved_session_file_path = g_file_get_path (saved_session_file);
+	crashed_session = g_file_test (saved_session_file_path, G_FILE_TEST_EXISTS);
+	
+	g_free (saved_session_file_path);
 
 	if (crashed_session == FALSE ||
 	    priv->windows != NULL ||
@@ -647,7 +654,7 @@ session_command_open_bookmarks_editor (EphySession *session,
 	GtkWidget *editor;
 
 	editor = ephy_shell_get_bookmarks_editor (ephy_shell_get_default ());
-        
+	
 	gtk_window_present_with_time (GTK_WINDOW (editor), user_time);
 }
 
@@ -659,7 +666,7 @@ session_command_open_uris (EphySession *session,
 {
 	EphyShell *shell;
 	EphyWindow *window;
-	EphyTab *tab;
+	EphyEmbed *embed;
 	EphyNewTabFlags flags = 0;
 	guint i;
 
@@ -669,6 +676,10 @@ session_command_open_uris (EphySession *session,
 
 	window = ephy_session_get_active_window (session);
 
+	if (options != NULL && strstr (options, "external") != NULL)
+	{
+		flags |= EPHY_NEW_TAB_FROM_EXTERNAL;
+	}
 	if (options != NULL && strstr (options, "new-window") != NULL)
 	{
 		window = NULL;
@@ -694,15 +705,15 @@ session_command_open_uris (EphySession *session,
 			page_flags = EPHY_NEW_TAB_OPEN_PAGE;
 		}
 
-		tab = ephy_shell_new_tab_full (shell, window,
-					       NULL /* parent tab */,
-					       url,
-					       flags | page_flags,
-					       EPHY_EMBED_CHROME_ALL,
-					       FALSE /* is popup? */,
-					       user_time);
+		embed = ephy_shell_new_tab_full (shell, window,
+						 NULL /* parent tab */,
+						 url,
+						 flags | page_flags,
+						 EPHY_EMBED_CHROME_ALL,
+						 FALSE /* is popup? */,
+						 user_time);
 
-		window = EPHY_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (tab)));
+		window = EPHY_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (embed)));
 	}
 
 	g_object_unref (shell);
@@ -983,8 +994,8 @@ ephy_session_class_init (EphySessionClass *class)
 		(object_class,
 		 PROP_ACTIVE_WINDOW,
 		 g_param_spec_object ("active-window",
-		 		      "Active Window",
-		 		      "The active window",
+				      "Active Window",
+				      "The active window",
 				      EPHY_TYPE_WINDOW,
 				      G_PARAM_READABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
 
@@ -1060,7 +1071,7 @@ ephy_session_close (EphySession *session)
 
 static int
 write_tab (xmlTextWriterPtr writer,
-	   EphyTab *tab)
+	   EphyEmbed *embed)
 {
 	const char *address, *title;
 	int ret;
@@ -1068,17 +1079,17 @@ write_tab (xmlTextWriterPtr writer,
 	ret = xmlTextWriterStartElement (writer, (xmlChar *) "embed");
 	if (ret < 0) return ret;
 
-	address = ephy_tab_get_address (tab);
+	address = ephy_embed_get_address (embed);
 	ret = xmlTextWriterWriteAttribute (writer, (xmlChar *) "url",
 					   (const xmlChar *) address);
 	if (ret < 0) return ret;
 
-	title = ephy_tab_get_title (tab);
+	title = ephy_embed_get_title (embed);
 	ret = xmlTextWriterWriteAttribute (writer, (xmlChar *) "title",
 					   (const xmlChar *) title);
 	if (ret < 0) return ret;
 
-	if (ephy_tab_get_load_status (tab))
+	if (ephy_embed_get_load_status (embed))
 	{
 		ret = xmlTextWriterWriteAttribute (writer,
 						   (const xmlChar *) "loading",
@@ -1099,7 +1110,7 @@ write_active_tab (xmlTextWriterPtr writer,
 
 	current = gtk_notebook_get_current_page (GTK_NOTEBOOK (notebook));
     
-    	ret = xmlTextWriterWriteFormatAttribute (writer, (const xmlChar *) "active-tab", "%d", current);
+	ret = xmlTextWriterWriteFormatAttribute (writer, (const xmlChar *) "active-tab", "%d", current);
 	return ret;
 }
     
@@ -1171,8 +1182,8 @@ write_ephy_window (xmlTextWriterPtr writer,
 	const char *role;
 	int ret;
 
-	tabs = ephy_window_get_tabs (window);
-    	notebook = ephy_window_get_notebook (window);
+	tabs = ephy_embed_container_get_children (EPHY_EMBED_CONTAINER (window));
+	notebook = ephy_window_get_notebook (window);
 
 	/* Do not save an empty EphyWindow.
 	 * This only happens when the window was newly opened.
@@ -1185,7 +1196,7 @@ write_ephy_window (xmlTextWriterPtr writer,
 	ret = write_window_geometry (writer, GTK_WINDOW (window));
 	if (ret < 0) return ret;
 
-    	ret = write_active_tab (writer, notebook);
+	ret = write_active_tab (writer, notebook);
 	if (ret < 0) return ret;
 
 	role = gtk_window_get_role (GTK_WINDOW (window));
@@ -1199,8 +1210,8 @@ write_ephy_window (xmlTextWriterPtr writer,
 
 	for (l = tabs; l != NULL; l = l->next)
 	{
-		EphyTab *tab = EPHY_TAB(l->data);
-		ret = write_tab (writer, tab);
+		EphyEmbed *embed = EPHY_EMBED (l->data);
+		ret = write_tab (writer, embed);
 		if (ret < 0) break;
 	}
 	g_list_free (tabs);
@@ -1214,12 +1225,17 @@ gboolean
 ephy_session_save (EphySession *session,
 		   const char *filename)
 {
-	EphySessionPrivate *priv = session->priv;
+	EphySessionPrivate *priv;
 	xmlTextWriterPtr writer;
 	GList *w;
-	char *save_to, *tmp_file;
+	GFile *save_to_file, *tmp_file;
+	char *tmp_file_path;
 	int ret;
 
+	g_return_val_if_fail (EPHY_IS_SESSION (session), FALSE);
+
+	priv = session->priv;
+
 	if (priv->dont_save)
 	{
 		return TRUE;
@@ -1233,15 +1249,15 @@ ephy_session_save (EphySession *session,
 		return TRUE;
 	}
 
-	save_to = get_session_filename (filename);
-	tmp_file = g_strconcat (save_to, ".tmp", NULL);
+	save_to_file = get_session_file (filename);
+	tmp_file_path = g_strconcat (g_file_get_path (save_to_file), ".tmp", NULL);
+	tmp_file = g_file_new_for_path (tmp_file_path);
 
 	/* FIXME: do we want to turn on compression? */
-	writer = xmlNewTextWriterFilename (tmp_file, 0);
+	writer = xmlNewTextWriterFilename (tmp_file_path, 0);
 	if (writer == NULL)
 	{
-		g_free (save_to);
-		g_free (tmp_file);
+		g_free (tmp_file_path);
 
 		return FALSE;
 	}
@@ -1249,7 +1265,7 @@ ephy_session_save (EphySession *session,
 	ret = xmlTextWriterSetIndent (writer, 1);
 	if (ret < 0) goto out;
 
-	ret = xmlTextWriterSetIndentString (writer, (const xmlChar *) "  ");
+	ret = xmlTextWriterSetIndentString (writer, (const xmlChar *) "	 ");
 	if (ret < 0) goto out;
 
 	START_PROFILER ("Saving session")
@@ -1284,14 +1300,15 @@ out:
 
 	if (ret >= 0)
 	{
-		if (ephy_file_switch_temp_file (save_to, tmp_file) == FALSE)
+		if (ephy_file_switch_temp_file (save_to_file, tmp_file) == FALSE)
 		{
 			ret = -1;
 		}
 	}
 
-	g_free (save_to);
-	g_free (tmp_file);
+	g_free (tmp_file_path);
+	g_object_unref (save_to_file);
+	g_object_unref (tmp_file);
 
 	STOP_PROFILER ("Saving session")
 
@@ -1333,9 +1350,11 @@ parse_embed (xmlNodePtr child,
 				char *escaped_url, *escaped_title;
 
 				title = xmlGetProp (child, (const xmlChar *) "title");
-				escaped_title = gnome_vfs_escape_string (title ? (const char*) title : _("Untitled"));
+				escaped_title = g_uri_escape_string (title ? (const char*) title : _("Untitled"),
+								     NULL, TRUE);
 
-				escaped_url = gnome_vfs_escape_string ((const char *) url);
+				escaped_url = g_uri_escape_string ((const char *) url,
+								   NULL, TRUE);
 				freeme = recover_url =
 					g_strconcat ("about:recover?u=",
 						     escaped_url,
@@ -1368,6 +1387,8 @@ int_from_string (const char *string,
 	long int val;
 	gboolean success = FALSE;
 
+	if (string == NULL) return FALSE;
+
 	errno = 0;
 	val = strtol (string, &tail, 0);
 
@@ -1435,17 +1456,19 @@ ephy_session_load (EphySession *session,
 {
 	EphySessionPrivate *priv = session->priv;
 	xmlDocPtr doc;
-        xmlNodePtr child;
+	xmlNodePtr child;
 	EphyWindow *window;
 	GtkWidget *widget = NULL;
-	char *save_to;
+	GFile *save_to_file;
+	char *save_to_path;
 
 	LOG ("ephy_sesion_load %s", filename);
 
-	save_to = get_session_filename (filename);
+	save_to_file = get_session_file (filename);
+	save_to_path = g_file_get_path (save_to_file);
 
-	doc = xmlParseFile (save_to);
-	g_free (save_to);
+	doc = xmlParseFile (save_to_path);
+	g_free (save_to_path);
 
 	if (doc == NULL)
 	{
@@ -1465,7 +1488,7 @@ ephy_session_load (EphySession *session,
 	{
 		if (xmlStrEqual (child->name, (const xmlChar *) "window"))
 		{
-		    	xmlChar *tmp;
+			xmlChar *tmp;
 		    
 			window = ephy_window_new ();
 			widget = GTK_WIDGET (window);
@@ -1493,7 +1516,8 @@ ephy_session_load (EphySession *session,
 				}
 			}
 
-			gtk_widget_grab_focus (GTK_WIDGET (ephy_window_get_active_tab (window)));
+			gtk_widget_grab_focus (GTK_WIDGET (ephy_embed_container_get_active_child
+                                                           (EPHY_EMBED_CONTAINER (window))));
 			gtk_widget_show (widget);
 		}
 		else if (xmlStrEqual (child->name, (const xmlChar *) "toolwindow"))
@@ -1605,18 +1629,19 @@ ephy_session_remove_window (EphySession *session,
 EphyWindow *
 ephy_session_get_active_window (EphySession *session)
 {
-	EphyWindow *window = NULL, *w;
+        EphyWindow *window = NULL;
+        EphyEmbedContainer *w;
 	GList *l;
 
 	g_return_val_if_fail (EPHY_IS_SESSION (session), NULL);
 
 	for (l = session->priv->windows; l != NULL; l = l->next)
 	{
-		w = EPHY_WINDOW (l->data);
+		w = EPHY_EMBED_CONTAINER (l->data);
 
-		if (ephy_window_get_is_popup (w) == FALSE)
+		if (ephy_embed_container_get_is_popup (w) == FALSE)
 		{
-			window = w;
+                        window = EPHY_WINDOW (w);
 			break;
 		}
 	}
