@@ -14,9 +14,9 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- *  $Id: ephy-shell.c,v 1.148 2006/11/12 21:03:41 jframeau Exp $
+ *  $Id: ephy-shell.c 7856 2008-01-10 20:40:35Z cosimoc $
  */
 
 #include "config.h"
@@ -29,6 +29,7 @@
 
 #include "ephy-shell.h"
 #include "ephy-type-builtins.h"
+#include "ephy-embed-container.h"
 #include "ephy-embed-shell.h"
 #include "ephy-embed-single.h"
 #include "eel-gconf-extensions.h"
@@ -52,6 +53,8 @@
 #include "ephy-toolbar.h"
 #include "ephy-prefs.h"
 #include "ephy-gui.h"
+#include "ephy-stock-icons.h"
+#include "ephy-embed-factory.h"
 
 #ifdef ENABLE_NETWORK_MANAGER
 #include "ephy-net-monitor.h"
@@ -140,7 +143,6 @@ ephy_shell_new_window_cb (EphyEmbedSingle *single,
 			  EphyShell *shell)
 {
 	GtkWidget *parent = NULL;
-	EphyTab *new_tab;
 	gboolean is_popup;
 	EphyNewTabFlags flags = EPHY_NEW_TAB_DONT_SHOW_WINDOW |
 				EPHY_NEW_TAB_APPEND_LAST |
@@ -168,12 +170,10 @@ ephy_shell_new_window_cb (EphyEmbedSingle *single,
 	 */
 	is_popup = (chromemask & EPHY_EMBED_CHROME_MENUBAR) == 0;
 
-	new_tab = ephy_shell_new_tab_full
+	return ephy_shell_new_tab_full
 		(shell,
 		 EPHY_IS_WINDOW (parent) ? EPHY_WINDOW (parent) : NULL,
 		 NULL, NULL, flags, chromemask, is_popup, 0);
-
-	return ephy_tab_get_embed (new_tab);
 }
 
 
@@ -199,7 +199,7 @@ ephy_shell_add_sidebar_cb (EphyEmbedSingle *embed_single,
 					 _("Sidebar extension required"));
 
 	gtk_window_set_title (GTK_WINDOW (dialog), _("Sidebar Extension Required"));
-	gtk_window_set_icon_name (GTK_WINDOW (dialog), "web-browser");
+	gtk_window_set_icon_name (GTK_WINDOW (dialog), EPHY_STOCK_EPHY);
 
 	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
 		  _("The link you clicked needs the sidebar extension to be "
@@ -436,7 +436,7 @@ load_homepage (EphyEmbed *embed)
  * ephy_shell_new_tab_full:
  * @shell: a #EphyShell
  * @parent_window: the target #EphyWindow or %NULL
- * @previous_tab: the referrer tab or %NULL
+ * @previous_embed: the referrer embed, or %NULL
  * @url: an url to load or %NULL
  * @chrome: a #EphyEmbedChrome mask to use if creating a new window
  * @is_popup: whether the new window is a popup
@@ -445,12 +445,12 @@ load_homepage (EphyEmbed *embed)
  * Create a new tab and the parent window when necessary.
  * Use this function to open urls in new window/tabs.
  *
- * ReturnValue: the created #EphyTab
+ * ReturnValue: the created #EphyEmbed
  **/
-EphyTab *
+EphyEmbed *
 ephy_shell_new_tab_full (EphyShell *shell,
 			 EphyWindow *parent_window,
-			 EphyTab *previous_tab,
+			 EphyEmbed *previous_embed,
 			 const char *url,
 			 EphyNewTabFlags flags,
 			 EphyEmbedChrome chrome,
@@ -458,11 +458,10 @@ ephy_shell_new_tab_full (EphyShell *shell,
 			 guint32 user_time)
 {
 	EphyWindow *window;
-	EphyTab *tab;
-	EphyEmbed *embed;
+	EphyEmbed *embed = NULL;
 	gboolean in_new_window = TRUE;
 	gboolean jump_to;
-	EphyEmbed *previous_embed = NULL;
+	gboolean active_is_blank = FALSE;
 	GtkWidget *nb;
 	int position = -1;
 	gboolean is_empty = FALSE;
@@ -476,8 +475,8 @@ ephy_shell_new_tab_full (EphyShell *shell,
 
 	jump_to = (flags & EPHY_NEW_TAB_JUMP) != 0;
 
-	LOG ("Opening new tab parent-window %p parent-tab %p in-new-window:%s jump-to:%s",
-	     parent_window, previous_tab, in_new_window ? "t" : "f", jump_to ? "t" : "f");
+	LOG ("Opening new tab parent-window %p parent-embed %p in-new-window:%s jump-to:%s",
+	     parent_window, previous_embed, in_new_window ? "t" : "f", jump_to ? "t" : "f");
 
 	if (!in_new_window && parent_window != NULL)
 	{
@@ -490,32 +489,43 @@ ephy_shell_new_tab_full (EphyShell *shell,
 
 	toolbar = EPHY_TOOLBAR (ephy_window_get_toolbar (window));
 
-	if (previous_tab != NULL)
-	{
-		previous_embed = ephy_tab_get_embed (previous_tab);
-	}
-
-	if ((flags & EPHY_NEW_TAB_APPEND_AFTER) && previous_tab != NULL)
+	if ((flags & EPHY_NEW_TAB_APPEND_AFTER) && previous_embed != NULL)
 	{
 		nb = ephy_window_get_notebook (window);
+                /* FIXME this assumes the tab is the direct notebook child */
 		position = gtk_notebook_page_num (GTK_NOTEBOOK (nb),
-						  GTK_WIDGET (previous_tab)) + 1;
+						  GTK_WIDGET (previous_embed)) + 1;
+	}
+	
+	if (flags & EPHY_NEW_TAB_FROM_EXTERNAL)
+	{
+		/* If the active embed is blank, use that to open the url and jump to it */
+		embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
+		if (embed != NULL)
+		{
+			if (ephy_embed_get_is_blank (embed))
+			{
+				active_is_blank = TRUE;
+			}
+		}
+	}
+	if (active_is_blank == FALSE)
+	{
+		embed = EPHY_EMBED (ephy_embed_factory_new_object (EPHY_TYPE_EMBED));
+		g_assert (embed != NULL);
+		gtk_widget_show (GTK_WIDGET (embed));
+		
+		ephy_embed_container_add_child (EPHY_EMBED_CONTAINER (window), embed, position, jump_to);
 	}
 
-	tab = ephy_tab_new ();
-	gtk_widget_show (GTK_WIDGET (tab));
-	embed = ephy_tab_get_embed (tab);
-
-	ephy_window_add_tab (window, tab, position, jump_to);
-
 	if (previous_embed != NULL)
-	{
+	{	
 		ephy_embed_shistory_copy (previous_embed,
 					  embed,
 					  TRUE,   /* back history */
 					  TRUE,   /* forward history */
 					  FALSE); /* current index */
-	}
+	}		
 
 	ephy_gui_window_update_user_time (GTK_WIDGET (window), user_time);
 
@@ -532,8 +542,8 @@ ephy_shell_new_tab_full (EphyShell *shell,
 	if (flags & EPHY_NEW_TAB_HOME_PAGE ||
 	    flags & EPHY_NEW_TAB_NEW_PAGE)
 	{
-		ephy_tab_set_typed_address (tab, "",
-					    EPHY_TAB_ADDRESS_EXPIRE_NEXT);
+		ephy_embed_set_typed_address (embed, "",
+                                              EPHY_EMBED_ADDRESS_EXPIRE_NEXT);
 		ephy_toolbar_activate_location (toolbar);
 		is_empty = load_homepage (embed);
 	}
@@ -581,30 +591,30 @@ ephy_shell_new_tab_full (EphyShell *shell,
                 }
         }
 
-	return tab;
+	return embed;
 }
 
 /**
  * ephy_shell_new_tab:
  * @shell: a #EphyShell
  * @parent_window: the target #EphyWindow or %NULL
- * @previous_tab: the referrer tab or %NULL
+ * @previous_embed: the referrer embed, or %NULL
  * @url: an url to load or %NULL
  *
  * Create a new tab and the parent window when necessary.
  * Use this function to open urls in new window/tabs.
  *
- * ReturnValue: the created #EphyTab
+ * ReturnValue: the created #EphyEmbed
  **/
-EphyTab *
+EphyEmbed *
 ephy_shell_new_tab (EphyShell *shell,
 		    EphyWindow *parent_window,
-		    EphyTab *previous_tab,
+		    EphyEmbed *previous_embed,
 		    const char *url,
 		    EphyNewTabFlags flags)
 {
 	return ephy_shell_new_tab_full (shell, parent_window,
-					previous_tab, url, flags,
+					previous_embed, url, flags,
 					EPHY_EMBED_CHROME_ALL, FALSE, 0);
 }
 
